<?php

/**
 *
 * @file
 * Multiple Field Question extend for quiz question.
 */
class MultipleFieldQuestion extends QuizQuestion {

  /**
   * Implements save.
   * Stores the question in the database.
   *
   * @param 
   * is_new if - if the node is a new node.
   *
   */
  public function saveNodeProperties($is_new = FALSE) {
    $is_new = $is_new || $this->node->revision == 1;
    // If new quiz insert values.
    if ($is_new) {
      db_insert('quiz_multiple_field_question_node_properties')
          ->fields(array(
            'nid' => $this->node->nid,
            'vid' => $this->node->vid,
            'type' => 1,
          ))
          ->execute();
    }
    // Update if not new quiz.
    else {
      db_update('quiz_multiple_field_question_node_properties')
          ->fields(array(
            'type' => 1,
          ))
          ->condition('nid', $this->node->nid)
          ->condition('vid', $this->node->vid)
          ->execute();
    }
  }

  /**
   * Implements validate.
   * QuizQuestion#validate().
   *
   */
  public function validateNode(array &$form) {
    
  }

  /**
   * Implements delete.
   *
   * @see QuizQuestion#delete().
   */
  public function delete($only_this_version = FALSE) {
    // Delete values if Quiz Webform deletes.
    if ($only_this_version) {
      db_delete('quiz_multiple_field_question_node_properties')
          ->condition('question_nid', $this->node->nid)
          ->condition('question_vid', $this->node->vid)
          ->execute();
      db_delete('quiz_multiple_field_question_user_answers')
          ->condition('nid', $this->node->nid)
          ->condition('vid', $this->node->vid)
          ->execute();
    }
    else {
      db_delete('quiz_multiple_field_question_node_properties')
          ->condition('nid', $this->node->nid)
          ->execute();
      db_delete('quiz_multiple_field_question_user_answers')
          ->condition('question_nid', $this->node->nid)
          ->execute();
    }
    parent::delete($only_this_version);
  }

  /**
   * Implements getNodeView.
   */
  public function getNodeView() {
    $content = parent::getNodeView();
    return $content;
  }

  /**
   * Implements getNodeProperties.
   */
  public function getNodeProperties() {
    if (isset($this->nodeProperties)) {
      return $this->nodeProperties;
    }
    $props = parent::getNodeProperties();
    $this->nodeProperties = $props;
    return $props;
  }

  /**
   * Generates the question form.
   *
   * This is called whenever a question is rendered, either
   * to an administrator or to a quiz taker.
   */
  public function getAnsweringForm(array $form_state = NULL, $rid) {
    module_load_include('inc', 'form_builder', 'includes/form_builder.admin');
    $data = form_builder_interface('multiple_field_question', $form_state['build_info']['args'][0]->nid);
    $values = '';
    foreach ($data[0] as $key => $value) {
      if ((strpos($key, 'sample_') !== FALSE)) {
        unset($data[0][$key]);
      }
      if (strpos($key, 'new_') !== FALSE) {
        if ($value['#type'] == 'checkboxes') {
          $values[$key] = $value;
        }
        elseif ($value['#type'] == 'radios') {
          $values[$key] = $value;
        }
        elseif ($value['#type'] == 'fieldset') {
          $fieldset = variable_get('fieldset_inline_' . arg(1), 0);
          $hide_labels = variable_get('hide_labels_' . arg(1), 0);
          if ($fieldset == 1) {
            $value['#attributes'] = array('class' => array('container-inline'));
          }
          if ($hide_labels == 1) {
            foreach ($value as $key1 => $value1) {
              if (strpos($key1, 'new_') !== FALSE) {
                $value[$key1]['#title'] = '';
              }
              $values[$key] = $value;
            }
          }
          else {
            $values[$key] = $value;
          }
        }
        elseif ($value['#type'] == 'managed_file') {
          unset($value['#form_builder']);
          unset($value['#pre_render']);
          $form[$key] = array(
            '#weight' => '-1',
            '#size' => '30',
            '#type' => 'managed_file',
            '#title' => t($value['#title']),
            '#upload_location' => 'public://',
          );
        }
        else {
          unset($value['#form_builder']);
          unset($value['#pre_render']);
          $values[$key] = $value;
        }
      }
    }
    $form['tries'] = array();
    $op = drupal_render($values);
    drupal_add_js(drupal_get_path('module', 'multiple_field_question') . '/js/multiple_field_question.js');
    $form['tries'] = array(
      '#type' => 'markup',
      '#markup' => $op,
    );
    $form['question_type'] = array(
      '#type' => 'hidden',
      '#value' => 'quiz_multiple_field_question',
    );
    return $form;
  }

  /**
   * Implements getCreationForm.
   */
  public function getCreationForm(array &$form_state = NULL) {
    $form['question'] = array(
      '#type' => 'textarea',
      '#rows' => 2,
    );
    return $form;
  }

  /**
   * Implements getMaximumScore.
   *
   * @see QuizQuestion#getMaximumScore().
   */
  public function getMaximumScore() {
    return variable_get('quiz_multiple_field_question_default_score', 10);
  }

}

/**
 * Extension of QuizQuestionResponse
 */
class MultipleFieldResponse extends QuizQuestionResponse {

  /**
   * ID of the answer.
   */
  protected $answerId = 0;
  protected $file = NULL;
  protected $questionNode;

  /**
   * Constructor.
   */
  public function __construct($result_id, stdClass $question_node, $tries = NULL) {

    parent::__construct($result_id, $question_node, $tries);
    $tries = $_FILES;
    $this->answer = $tries;
    $this->answerFeedback = "";
    if (!isset($result) || !is_object($result)) {
      $result = new stdClass();
    }
    $result->is_correct = TRUE;
    $this->evaluated = 0;
    $this->questionNode = $question_node;
    $response = $this->getResponse();
    $this->result_id = $result_id;
    // Question has been answered already. We fetch the answer data from the database.
    $r = db_query('SELECT * FROM {quiz_multiple_field_question_user_answers}
    WHERE question_nid = :question_nid AND question_vid = :question_vid AND result_id = :result_id', array(
      ':question_nid' => $question_node->nid,
      ':question_vid' => $question_node->vid,
      ':result_id' => $result_id,
        ))->fetchAssoc();
    if (is_array($r)) {
      $this->score = $r['score'];
      $this->answerId = $r['answer_id'];
      $this->evaluated = $r['is_evaluated'];
      $this->answerFeedback = $r['answer_feedback'];
    }
  }

  /**
   * Implements isValid.
   */
  public function isValid() {
    module_load_include('inc', 'form_builder', 'includes/form_builder.admin');
    $data = form_builder_interface('multiple_field_question', $this->question->nid);
    $values = array();
    $f1 = 0;
    foreach ($_POST as $key => $value) {
      if (strpos($key, 'sample_') !== FALSE) {
        $values[$key] = $value;
        if (empty($value)) {
          $f1 = 1;
        }
      }
      if (strpos($key, 'new_') !== FALSE) {
        $values[$key] = $value;
        if (empty($value)) {
          $f1 = 1;
        }
        if (is_array($value)) {
          foreach ($value as $key3 => $value3) {
            $values[$key3] = $value3;
            if (empty($value3)) {
              $f1 = 1;
            }
          }
        }
        if ($data[0][$key]['#form_builder']['element_type'] == 'number') {
          if (!is_numeric($value)) {
            return t('Only Numeric Values allowed in ' . $data[0][$key]['#title']);
          }
        }
        if ($data[0][$key]['#form_builder']['element_type'] == 'email') {
          if (!filter_var($value, FILTER_VALIDATE_EMAIL)) {
            return t('Email not valid in ' . $data[0][$key]['#title']);
          }
        }
        if ($data[0][$key]['#form_builder']['element_type'] == 'fieldset') {
          foreach ($data[0][$key] as $key1 => $value1) {
            if (strpos($key1, 'new_') !== FALSE) {
              if ($data[0][$key][$key1]['#form_builder']['element_type'] == 'number') {
                if (!is_numeric($_POST[$key][$key1])) {
                  return t('Only Numeric Values allowed in ' . $data[0][$key][$key1]['#title']);
                }
              }
              if ($data[0][$key][$key1]['#form_builder']['element_type'] == 'email') {
                if (!filter_var($_POST[$key][$key1], FILTER_VALIDATE_EMAIL)) {
                  return t('Email not valid in ' . $data[0][$key][$key1]['#title']);
                }
              }
            }
          }
        }
      }
      if ($key == 'files') {
        $file_values = $values[$key];
        foreach ($file_values as $key2 => $value2) {
          $values[$key2] = $value2;
          if (empty($value2)) {
            $f1 = 1;
          }
        }
      }
    }
    if ($f1 == 1) {
      return t('Each field is required.');
    }
    $results = db_query('SELECT *
    FROM {quiz_multiple_field_question_user_answers} qf
    WHERE question_nid = :nid AND question_vid = :vid AND result_id = :result_id', array(':nid' => $this->questionNode->nid,
      ':vid' => $this->questionNode->vid, ':result_id' => $this->result_id)
        )
        ->fetchAssoc();
    if (!isset($results['result_id'])) {
      $answer_id = db_insert('quiz_multiple_field_question_user_answers')
          ->fields(array(
            'result_id' => $this->result_id,
            'question_vid' => $this->questionNode->vid,
            'question_nid' => $this->questionNode->nid,
            'multiple_field_question_id' => 0,
            'score' => 0,
            'is_evaluated' => 0,
            'answer' => serialize($values),
            'multiple_field_question_data' => serialize($_POST),
          ))
          ->execute();
    }
    else {
      $answer_id = db_update('quiz_multiple_field_question_user_answers')
          ->fields(array(
            'multiple_field_question_data' => serialize($_POST),
            'answer' => serialize($values),
          ))
          ->condition('result_id', $this->result_id)
          ->condition('question_vid', $this->questionNode->vid)
          ->condition('question_nid', $this->questionNode->nid)
          ->execute();
    }
    return TRUE;
  }

  /**
   * Implementation of save.
   */
  public function save() {
    $this->answerId = db_insert('quiz_multiple_field_question_user_answers')
        ->fields(array(
          'result_id' => $this->rid,
          'question_vid' => $this->question->vid,
          'question_nid' => $this->question->nid,
          'score' => $this->getScore(FALSE),
          'is_evaluated' => $this->evaluated,
          'answer_feedback' => $this->answerFeedback,
        ))
        ->execute();
  }

  /**
   * Implementation of delete.
   */
  public function delete() {
    db_delete('quiz_multiple_field_question_user_answers')
        ->condition('question_nid', $this->question->nid)
        ->condition('question_vid', $this->question->vid)
        ->condition('result_id', $this->rid)
        ->execute();
  }

  /**
   * Implements score.
   *
   * @return uint.
   */
  public function score() {
    return (int) db_query('SELECT score FROM {quiz_multiple_field_question_user_answers}
      WHERE result_id = :result_id AND question_vid = :question_vid', array(':result_id' => $this->rid, ':question_vid' => $this->question->vid))->fetchField();
  }

  /**
   * Implements getResponse.
   *
   * @return answer.
   */
  public function getResponse() {
    return $this->answer;
  }

  public function getReportFormResponse($showpoints = TRUE, $showfeedback = TRUE, $allow_scoring = FALSE) {
    $result_id = $this->question->answers[0]['result_id'];
    $form = array();
    $rs = db_query('SELECT ua.*, n.title FROM {quiz_multiple_field_question_user_answers} AS ua
      LEFT JOIN {node} AS n ON n.nid = ua.question_nid
      WHERE ua.result_id = :result_id AND ua.question_nid = :question_nid AND ua.question_vid = :question_vid', array(
      ':result_id' => $result_id,
      ':question_nid' => $this->question->nid,
      ':question_vid' => $this->question->vid,
    ));
    // Set values to display response.
    module_load_include('inc', 'form_builder', 'includes/form_builder.admin');
    $data = form_builder_interface('multiple_field_question', $this->question->nid);
    $values = array();
    foreach ($rs as $result) {
      $answers = unserialize($result->answer);
      $i = 1;
      $check_value = '';
      foreach ($data[0] as $key => $value) {
        if ((strpos($key, 'new_') !== FALSE)) {
          if (($value['#type'] != 'checkboxes') && ($value['#type'] != 'date') && ($value['#type'] != 'fieldset') && ($value['#type'] != 'managed_file')) {
            $anwser_value = '';
            if (isset($value['#field_prefix'])) {
              $anwser_value .= $value['#field_prefix'] . ' ';
            }
            if (!empty($answers[$key])) {
              $anwser_value .= $answers[$key];
            }
            if (isset($value['#field_suffix'])) {
              $anwser_value .= ' ' . $value['#field_suffix'];
            }
            if (!empty($anwser_value)) {
              $rows['r' . $i] = array(
                'c1' => $value['#title'],
                'c2' => $anwser_value,
              );
            }
          }
          elseif ($value['#type'] == 'fieldset') {
            $rows['r' . $i] = array(
              'c1' => 'Fieldset', 'c2' => ''
            );
            foreach ($answers[$key] as $key1 => $value1) {
              if (($value[$key1]['#type'] != 'checkboxes') && ($value[$key1]['#type'] != 'date') && ($value[$key1]['#type'] != 'fieldset') && ($value[$key1]['#type'] != 'managed_file')) {
                $anwser_value = '';
                if (isset($value[$key1]['#field_prefix'])) {
                  $anwser_value .= $value[$key1]['#field_prefix'] . ' ';
                }
                if (!empty($answers[$key])) {
                  $anwser_value .= $value1;
                }
                if (isset($value[$key1]['#field_suffix'])) {
                  $anwser_value .= ' ' . $value[$key1]['#field_suffix'];
                }
                if (!empty($anwser_value)) {
                  $rows['r' . $i] = array(
                    'c1' => $value[$key1]['#title'],
                    'c2' => $anwser_value,
                  );
                }
              }
              elseif ($value[$key1]['#type'] == 'managed_file') {
                $file_name = $value[$key1]['#key'];
                $fid = $value1['fid'];
                $file_load = file_load($fid);
                $file_path = file_create_url($file_load->uri);
                $rows['r' . $i] = array(
                  'c1' => $value['#title'],
                  'c2' => '<a href="' . $file_path . '" target="_blank">' . $file_load->filename . '</a>',
                );
              }
              elseif ($value[$key1]['#type'] == 'date') {
                $anwser_value = $value1['month'] . '/' . $value1['day'] . '/' . $value1['year'] . ' (M/D/Y)';
                if ($anwser_value != '// (M/D/Y)') {
                  $rows['r' . $i] = array(
                    'c1' => $value[$key1]['#title'],
                    'c2' => $anwser_value,
                  );
                }
              }
              // For Checkbox.
              else {
                foreach ($answers[$key] as $key1 => $value1) {
                  $check_value .= $value1 . ', ';
                }
                if ($check_value != ', ') {
                  $rows['r' . $i] = array(
                    'c1' => $value[$key1]['#title'],
                    'c2' => rtrim($check_value, ', '),
                  );
                }
              }
              $i++;
            }
          }
          elseif ($value['#type'] == 'managed_file') {
            $file_name = $value['#key'];
            $anwser_value = $answers[$file_name]['fid'];
            $file_load = file_load($answers[$file_name]['fid']);
            $file_path = file_create_url($file_load->uri);
            $rows['r' . $i] = array(
              'c1' => $value['#title'],
              'c2' => '<a href="' . $file_path . '" target="_blank">' . $file_load->filename . '</a>',
            );
          }
          elseif ($value['#type'] == 'date') {
            $anwser_value = $answers[$key]['month'] . '/' . $answers[$key]['day'] . '/' . $answers[$key]['year'] . ' (M/D/Y)';
            if ($anwser_value != '// (M/D/Y)') {
              $rows['r' . $i] = array(
                'c1' => $value['#title'],
                'c2' => $anwser_value,
              );
            }
          }
          else {
            foreach ($answers[$key] as $key1 => $value1) {
              $check_value .= $value1 . ', ';
            }
            if ($check_value != ', ') {
              $rows['r' . $i] = array(
                'c1' => $value['#title'],
                'c2' => rtrim($check_value, ', '),
              );
            }
          }
          $i++;
        }
      }
    }
    if (!empty($rows)) {
      $form['submitted_data'] = array(
        '#theme' => 'table',
        '#header' => array(t('Field Name'), t('Answer')),
        '#rows' => $rows,
      );
    }
    if ($this->question && !empty($this->question->answers)) {
      $answer = (object) current($this->question->answers);
    }
    if ($answer->is_evaluated == 1) {
      // Show feedback, if any.
      $form['answer_feedback'] = array(
        '#title' => t('Feedback'),
        '#type' => 'item',
        '#markup' => '<span class="quiz_answer_feedback">' . $this->answerFeedback . '</span>',
      );
    }
    else {
      $feedback = t('This answer has not yet been scored.') .
          '<br/>' .
          t('Until the answer is scored, the total score will not be correct.');
    }
    return $form;
  }

  /**
   * Getting response of quiz.
   */
  public function getReportFormScore($showfeedback = TRUE, $showpoints = TRUE, $allow_scoring = FALSE) {
    $node = node_load($this->question->nid);
    $score = ($this->isEvaluated()) ? $this->getScore() : '?';
    if (quiz_access_to_score() && $allow_scoring) {
      return array(
        '#type' => 'textfield',
        '#default_value' => $score,
        '#size' => 3,
        '#maxlength' => 3,
        '#attributes' => array('class' => array('quiz-report-score')),
      );
    }
    else {
      return array(
        '#markup' => $score,
      );
    }
  }

  /**
   * Getting Feedback of quiz.
   */
  public function getReportFormAnswerFeedback($showpoints = TRUE, $showfeedback = TRUE, $allow_scoring = FALSE) {
    if (quiz_access_to_score() && $allow_scoring) {
      return array(
        '#title' => t('Enter feedback'),
        '#type' => 'textarea',
        '#default_value' => $this->answerFeedback,
        '#attributes' => array('class' => array('quiz-report-score')),
      );
    }
    return FALSE;
  }

  /**
   * Implements getReportFormSubmit.
   */
  public function getReportFormSubmit($showfeedback = TRUE, $showpoints = TRUE, $allow_scoring = FALSE) {
    return $allow_scoring ? 'multiple_field_question_report_submit' : FALSE;
  }

  /**
   * Implements getReportFormValidate.
   */
  public function getReportFormValidate($showfeedback = TRUE, $showpoints = TRUE, $allow_scoring = FALSE) {
    return $allow_scoring ? 'multiple_field_question_report_validate' : FALSE;
  }

}
